﻿using ProjectSportCar.Entities;

namespace ProjectSportCar.Drawnings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>
public class DrawningCar
{
	/// <summary>
	/// Класс-сущность
	/// </summary>
	public EntityCar? EntityCar { get; protected set; }

	/// <summary>
	/// Ширина окна
	/// </summary>
	private int? _pictureWidth;

	/// <summary>
	/// Высота окна
	/// </summary>
	private int? _pictureHeight;

	/// <summary>
	/// Левая координата прорисовки автомобиля
	/// </summary>
	protected int? _startPosX;

	/// <summary>
	/// Верхняя кооридната прорисовки автомобиля
	/// </summary>
	protected int? _startPosY;

	/// <summary>
	/// Ширина прорисовки автомобиля
	/// </summary>
	private readonly int _drawningCarWidth = 90;

	/// <summary>
	/// Высота прорисовки автомобиля
	/// </summary>
	private readonly int _drawningCarHeight = 50;

	/// <summary>
	/// Координата X объекта
	/// </summary>
	public int? GetPosX => _startPosX;

	/// <summary>
	/// Координата Y объекта
	/// </summary>
	public int? GetPosY => _startPosY;

	/// <summary>
	/// Ширина объекта
	/// </summary>
	public int GetWidth => _drawningCarWidth;

	/// <summary>
	/// Высота объекта
	/// </summary>
	public int GetHeight => _drawningCarHeight;

	/// <summary>
	/// Пустой конструктор
	/// </summary>
	private DrawningCar()
	{
		_pictureWidth = null;
		_pictureHeight = null;
		_startPosX = null;
		_startPosY = null;
	}

	/// <summary>
	/// Конструктор
	/// </summary>
	/// <param name="speed">Скорость</param>
	/// <param name="weight">Вес</param>
	/// <param name="bodyColor">Основной цвет</param>
	public DrawningCar(int speed, double weight, Color bodyColor) : this()
	{
		EntityCar = new EntityCar(speed, weight, bodyColor);
	}

	/// <summary>
	/// Конструктор для наследников
	/// </summary>
	/// <param name="drawningCarWidth">Ширина прорисовки автомобиля</param>
	/// <param name="drawningCarHeight">Высота прорисовки автомобиля</param>
	protected DrawningCar(int drawningCarWidth, int drawningCarHeight) : this()
	{
		_drawningCarWidth = drawningCarWidth;
		_pictureHeight = drawningCarHeight;
	}

	/// <summary>
	/// Установка границ поля
	/// </summary>
	/// <param name="width">Ширина поля</param>
	/// <param name="height">Высота поля</param>
	/// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
	public bool SetPictureSize(int width, int height)
	{
		// TODO проверка, что объект "влезает" в размеры поля
		// если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
		_pictureWidth = width;
		_pictureHeight = height;
		return true;
	}

	/// <summary>
	/// Установка позиции
	/// </summary>
	/// <param name="x">Координата X</param>
	/// <param name="y">Координата Y</param>
	public void SetPosition(int x, int y)
	{
		if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
		{
			return;
		}

		// TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
		// то надо изменить координаты, чтобы он оставался в этих границах
		_startPosX = x;
		_startPosY = y;
	}

	/// <summary>
	/// Изменение направления перемещения
	/// </summary>
	/// <param name="direction">Направление</param>
	/// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
	public bool MoveTransport(DirectionType direction)
	{
		if (EntityCar == null || !_startPosX.HasValue || !_startPosY.HasValue)
		{
			return false;
		}

		switch (direction)
		{
			//влево
			case DirectionType.Left:
				if (_startPosX.Value - EntityCar.Step > 0)
				{
					_startPosX -= (int)EntityCar.Step;
				}
				return true;
			//вверх
			case DirectionType.Up:
				if (_startPosY.Value - EntityCar.Step > 0)
				{
					_startPosY -= (int)EntityCar.Step;
				}
				return true;
			// вправо
			case DirectionType.Right:
				//TODO прописать логику сдвига в право
				return true;
			//вниз
			case DirectionType.Down:
				//TODO прописать логику сдвига в вниз
				return true;
			default:
				return false;
		}
	}

	/// <summary>
	/// Прорисовка объекта
	/// </summary>
	/// <param name="g"></param>
	public virtual void DrawTransport(Graphics g)
	{
		if (EntityCar == null || !_startPosX.HasValue || !_startPosY.HasValue)
		{
			return;
		}

		Pen pen = new(Color.Black);

		//границы автомобиля
		g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value, 20, 20);
		g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 30, 20, 20);
		g.DrawEllipse(pen, _startPosX.Value + 70, _startPosY.Value, 20, 20);
		g.DrawEllipse(pen, _startPosX.Value + 70, _startPosY.Value + 30, 20, 20);
		g.DrawRectangle(pen, _startPosX.Value - 1, _startPosY.Value + 10, 10, 30);
		g.DrawRectangle(pen, _startPosX.Value + 80, _startPosY.Value + 10, 10, 30);
		g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value - 1, 70, 52);

		//задние фары
		Brush brRed = new SolidBrush(Color.Red);
		g.FillEllipse(brRed, _startPosX.Value, _startPosY.Value, 20, 20);
		g.FillEllipse(brRed, _startPosX.Value, _startPosY.Value + 30, 20, 20);

		//передние фары
		Brush brYellow = new SolidBrush(Color.Yellow);
		g.FillEllipse(brYellow, _startPosX.Value + 70, _startPosY.Value, 20, 20);
		g.FillEllipse(brYellow, _startPosX.Value + 70, _startPosY.Value + 30, 20, 20);

		//кузов
		Brush br = new SolidBrush(EntityCar.BodyColor);
		g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 10, 10, 30);
		g.FillRectangle(br, _startPosX.Value + 80, _startPosY.Value + 10, 10, 30);
		g.FillRectangle(br, _startPosX.Value + 10, _startPosY.Value, 70, 50);

		//стекла
		Brush brBlue = new SolidBrush(Color.LightBlue);
		g.FillRectangle(brBlue, _startPosX.Value + 60, _startPosY.Value + 5, 5, 40);
		g.FillRectangle(brBlue, _startPosX.Value + 20, _startPosY.Value + 5, 5, 40);
		g.FillRectangle(brBlue, _startPosX.Value + 25, _startPosY.Value + 3, 35, 2);
		g.FillRectangle(brBlue, _startPosX.Value + 25, _startPosY.Value + 46, 35, 2);

		//выделяем рамкой крышу
		g.DrawRectangle(pen, _startPosX.Value + 25, _startPosY.Value + 5, 35, 40);
		g.DrawRectangle(pen, _startPosX.Value + 65, _startPosY.Value + 10, 25, 30);
		g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 10, 15, 30);
	}
}